@@ -87,6 +87,7 @@ class LabHandler(IPythonHandler):
         configData = dict(
             terminalsAvailable=self.settings.get('terminals_available', False),
         )
+        extension_prefix = ujoin(self.base_url, EXTENSION_PREFIX)
 
         # Gather the lab extension files and entry points.
         for (name, data) in sorted(labextensions.items()):
@@ -96,12 +97,12 @@ class LabHandler(IPythonHandler):
                 if value.get('entry', None):
                     entries.append(value['entry'])
                     bundles.append('%s/%s/%s' % (
-                        EXTENSION_PREFIX, name, value['files'][0]
+                        extension_prefix, name, value['files'][0]
                     ))
                 for fname in value['files']:
                     if os.path.splitext(fname)[1] == '.css':
                         css_files.append('%s/%s/%s' % (
-                            EXTENSION_PREFIX, name, fname
+                            extension_prefix, name, fname
                         ))
             python_module = data.get('python_module', None)
             if python_module:
@@ -131,6 +132,7 @@ default_handlers = [
 
 class LabApp(NotebookApp):
     version = __version__
+    name = 'jupyterlab'
 
     description = """
         JupyterLab - An extensible computational environment for Jupyter.
@@ -162,15 +164,15 @@ class LabApp(NotebookApp):
         """initialize tornado webapp and httpserver.
         """
         super(LabApp, self).init_webapp()
-        self.add_lab_handlers(self.web_app)
-        self.add_labextensions(self.web_app)
+        self.add_lab_handlers()
+        self.add_labextensions()
 
-    def add_labextensions(self, webapp):
+    def add_labextensions(self):
         """Get the enabledd and valid lab extensions.
 
         Notes
         -------
-        Adds a `labextensions` property to the webapp, which is a dict
+        Adds a `labextensions` property to the web_app, which is a dict
         containing manifest data for each enabled and active extension,
         and optionally its associated python_module.
         """
@@ -189,20 +191,22 @@ class LabApp(NotebookApp):
                 continue
             data['python_module'] = config.get('python_module', None)
             out[name] = data
-        webapp.labextensions = out
+        self.web_app.labextensions = out
 
-    def add_lab_handlers(self, webapp):
+    def add_lab_handlers(self):
         """Add the lab-specific handlers to the tornado app."""
-        base_url = webapp.settings['base_url']
-        webapp.add_handlers(".*$",
+        web_app = self.web_app
+        base_url = web_app.settings['base_url']
+        web_app.add_handlers(".*$",
             [(ujoin(base_url, h[0]),) + h[1:] for h in default_handlers])
+        extension_prefix = ujoin(base_url, EXTENSION_PREFIX)
         labextension_handler = (
-            r"%s/(.*)" % EXTENSION_PREFIX, FileFindHandler, {
+            r"%s/(.*)" % extension_prefix, FileFindHandler, {
                 'path': jupyter_path('labextensions'),
                 'no_cache_paths': ['/'],  # don't cache anything in labbextensions
             }
         )
-        webapp.add_handlers(".*$", [labextension_handler])
+        web_app.add_handlers(".*$", [labextension_handler])
         base_dir = os.path.realpath(os.path.join(HERE, '..'))
         dev_mode = os.path.exists(os.path.join(base_dir, '.git'))
         if dev_mode:
@@ -212,11 +216,29 @@ class LabApp(NotebookApp):
 def bootstrap_from_nbapp(nbapp):
     """Bootstrap the lab app on top of a notebook app.
     """
+    if isinstance(nbapp, LabApp):
+        return
+
     labapp = LabApp()
+
+    # Get data from the nbapp.
+    labapp.config_dir = nbapp.config_dir
+    labapp.log = nbapp.log
+    labapp.web_app = nbapp.web_app
+
+    cli_config = nbapp.cli_config.get('NotebookApp', {})
+    cli_config.update(nbapp.cli_config.get('LabApp', {}))
+
+    # Handle config.
+    for (key, value) in cli_config.items():
+        setattr(labapp, key, value)
     labapp.load_config_file()
-    webapp = nbapp.web_app
-    labapp.add_lab_handlers(webapp)
-    labapp.add_labextensions(webapp)
+    # Enforce cli override.
+    for (key, value) in cli_config.items():
+        setattr(labapp, key, value)
+
+    labapp.add_lab_handlers()
+    labapp.add_labextensions()
 
 
 #-----------------------------------------------------------------------------
